package com.example.civilizedcity.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.example.civilizedcity.common.BaseController;
import com.example.civilizedcity.common.Constants;
import com.example.civilizedcity.common.ResponseBean;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.example.civilizedcity.common.StringUtils;
import com.example.civilizedcity.entity.SysResource;
import com.example.civilizedcity.entity.SysRole;
import com.example.civilizedcity.service.SysResourceService;
import com.example.civilizedcity.service.SysRoleService;
import com.example.civilizedcity.service.SysUserDutyService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.example.civilizedcity.entity.SysUser;
import com.example.civilizedcity.service.SysUserService;

/**
 * 用户表;(sys_user)表控制层
 *
 * @author : http://njyunzhi.com
 * @date : 2022-12-12
 */
@Api(tags = "用户表对象功能接口")
@RestController
@RequestMapping("/")
public class SysUserController extends BaseController {

    @Autowired
    SysRoleService sysRoleService;

    @Autowired
    SysResourceService sysResourceService;

    @Autowired
    SysUserDutyService sysUserDutyService;

    /**
     * 通过ID查询单条数据
     *
     * @param userId 主键
     * @return 实例对象
     */
    @ApiOperation("通过ID查询单条数据")
    @GetMapping("/sysUser/{id}")
    public ResponseBean queryById(@ApiParam("对象ID") @PathVariable String id) throws Exception {
        SysUser sysUser = sysUserService.getById(id);
        if (null == sysUser || sysUser.getStatus() == Constants.STATUS_DELETE) {
            return ResponseBean.error("用户不存在");
        }

        return ResponseBean.success(getPermissions(sysUser));
    }

    /**
     * 查询当前人员
     *
     * @param
     * @return 实例对象
     */
    @ApiOperation("查询当前人员")
    @GetMapping("/sysUser/current")
    public ResponseBean current() throws Exception {
        SysUser sysUser = currentUser();
        return ResponseBean.success(getPermissions(sysUser));
    }

    private SysUser getPermissions(SysUser sysUser) {
        List<SysRole> rolesList = sysRoleService.getByUser(sysUser.getUserId());
        List<SysResource> resourcesList = sysResourceService.getByUser(sysUser.getUserId());
        List<String> dutyList = sysUserDutyService.getListByUser(sysUser.getUserId());

        sysUser.setRolesList(rolesList);
        sysUser.setResourcesList(resourcesList);
        sysUser.setDutyList(dutyList);

        return sysUser;
    }

    /**
     * 分页查询
     *
     * @param pageNum  当前页码
     * @param pageSize 每页条数
     * @return 查询结果
     */
    @ApiOperation("分页查询")
    @GetMapping("/sysUser")
    public ResponseBean list(@ApiParam("页码") @RequestParam(value = "pageNum", defaultValue = "1") Integer pageNum,
                             @ApiParam("单页数据量") @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize,
                             @ApiParam("姓名") @RequestParam(value = "name", required = false) String name,
                             @ApiParam("机构") @RequestParam(value = "orgId", required = false) String orgId,
                             @ApiParam("手机号") @RequestParam(value = "phone", required = false) String phone) throws Exception {

        IPage<SysUser> pg = new Page<>(pageNum, pageSize);
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.like(!StringUtils.isEmpty(name), "name", name);
        queryWrapper.eq(!StringUtils.isEmpty(orgId), "org_id", orgId);
        queryWrapper.like(!StringUtils.isEmpty(phone), "phone", phone);
        queryWrapper.eq("is_user", 1);
        queryWrapper.gt("status", Constants.STATUS_DELETE);
        queryWrapper.ne("user_id", Constants.ROOT_ID);
        queryWrapper.orderByDesc("create_date");
        IPage<SysUser> result = sysUserService.page(pg, queryWrapper);

        return ResponseBean.success(result);
    }

    /**
     * 新增数据
     *
     * @param sysUser 实例对象
     * @return 实例对象
     */
    @ApiOperation("新增数据")
    @PostMapping("/sysUser")
    public ResponseBean add(@ApiParam("对象实体") @RequestBody SysUser sysUser) throws Exception {
        if (null == sysUser.getOrgId()) {
            sysUser.setOrgId(currentUser().getOrgId());
        }

        sysUserService.mergeData(sysUser);
        return ResponseBean.success(sysUser);
    }

    /**
     * 更新数据
     *
     * @param id 实例ID
     * @return 实例对象
     */
    @ApiOperation("更新数据")
    @PutMapping("/sysUser/{id}/status")
    public ResponseBean edit(@ApiParam("对象ID") @PathVariable String id,
                             @ApiParam("页码") @RequestParam(value = "status") Integer status) throws Exception {

        SysUser sysUser = sysUserService.getById(id);
        sysUser.setStatus(status);

        sysUserService.updateById(sysUser);
        return ResponseBean.success(sysUser);
    }

//    /**
//     * 更新数据
//     *
//     * @param sysUser 实例对象
//     * @return 实例对象
//     */
//    @ApiOperation("更新数据")
//    @PutMapping("/sysUser/{id}")
//    public ResponseBean edit(@ApiParam("对象实体") @RequestBody SysUser sysUser,
//                            @ApiParam("对象ID") @PathVariable String id ) throws Exception {
//        sysUserService.updateById(sysUser);
//        return ResponseBean.success(sysUser);
//    }

    /**
     * 通过主键删除数据
     *
     * @param userId 主键
     * @return 是否成功
     */
    @ApiOperation("通过主键删除数据")
    @DeleteMapping("/sysUser/{id}")
    public ResponseBean deleteById(@ApiParam("对象ID") @PathVariable String id) {
        sysUserService.removeLogicById(id);
        return ResponseBean.success("success");
    }
}